# Generated by Django 3.2.4 on 2021-11-08 17:18

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('epic', '0009_auto_20191003_2243'),
    ]

    operations = [
        migrations.AlterField(
            model_name='assembly_item',
            name='comp',
            field=models.ForeignKey(help_text='The part number of the component of this item.', on_delete=django.db.models.deletion.PROTECT, related_name='assembly_item_comp', to='epic.part', verbose_name='Component Part #'),
        ),
        migrations.AlterField(
            model_name='delta',
            name='part',
            field=models.ForeignKey(help_text='The part whose quantity gets adjusted.', on_delete=django.db.models.deletion.PROTECT, to='epic.part', verbose_name='Part #'),
        ),
        migrations.AlterField(
            model_name='delta',
            name='warehouse',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='epic.warehouse'),
        ),
        migrations.AlterField(
            model_name='line_item',
            name='part',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='epic.part', verbose_name='Part #'),
        ),
        migrations.AlterField(
            model_name='order',
            name='vendor',
            field=models.ForeignKey(help_text='The name of the vendor (distributor) where the order was placed.', on_delete=django.db.models.deletion.PROTECT, to='epic.vendor'),
        ),
        migrations.AlterField(
            model_name='part',
            name='datasheet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='epic.datasheet'),
        ),
        migrations.AlterField(
            model_name='part',
            name='substitute',
            field=models.ForeignKey(blank=True, help_text='List of other parts that are (identical) substitutes for this part.', null=True, on_delete=django.db.models.deletion.PROTECT, to='epic.part', verbose_name='Substitutes'),
        ),
        migrations.AlterField(
            model_name='shipment',
            name='from_warehouse',
            field=models.ForeignKey(blank=True, help_text='For an inter-warehouse shipment, the warehouse the shipment originates from.', null=True, on_delete=django.db.models.deletion.PROTECT, to='epic.warehouse'),
        ),
        migrations.AlterField(
            model_name='shipment',
            name='ordr',
            field=models.ForeignKey(blank=True, help_text='For an order shipment, the order that resulted in this shipment.', null=True, on_delete=django.db.models.deletion.PROTECT, to='epic.order', verbose_name='Order #'),
        ),
        migrations.AlterField(
            model_name='stock_cache_item',
            name='part',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='epic.part', verbose_name='Part #'),
        ),
        migrations.AlterField(
            model_name='stock_cache_item',
            name='warehouse',
            field=models.ForeignKey(help_text='The warehouse stocking this part.', on_delete=django.db.models.deletion.PROTECT, to='epic.warehouse', verbose_name='Warehouse'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='warehouse',
            field=models.ForeignKey(help_text='The (destination) warehouse this transaction applies to.', on_delete=django.db.models.deletion.PROTECT, to='epic.warehouse', verbose_name='Warehouse'),
        ),
    ]
