# Generated by Django 2.0.2 on 2018-02-03 05:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epic', '0007_auto_20170510_0436'),
    ]

    operations = [
        migrations.AlterField(
            model_name='part',
            name='status',
            field=models.IntegerField(choices=[(0, 'preview'), (1, 'active'), (2, 'preferred'), (3, 'deprecated'), (4, 'obsolete')], default=0, help_text="The life-time status of this part. Parts marked `preview' and `obsolete' are not considered orderable.", verbose_name='Life-time Status'),
        ),
        migrations.AlterField(
            model_name='vendor_part',
            name='status',
            field=models.IntegerField(choices=[(0, 'preview'), (1, 'active'), (2, 'preferred'), (3, 'deprecated'), (4, 'obsolete')], default=1, help_text='The life-time status of this vendor part.', verbose_name='Life-time Status'),
        ),
    ]
