# Generated by Django 5.2.4 on 2025-08-01 02:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epic', '0012_alter_part_footprint'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='assembly_item',
            index=models.Index(fields=['assy', 'comp'], name='epic_assemb_assy_id_dcc6cf_idx'),
        ),
        migrations.AddIndex(
            model_name='line_item',
            index=models.Index(fields=['txtn', 'part'], name='epic_line_i_txtn_id_bbdfa8_idx'),
        ),
        migrations.AddIndex(
            model_name='part',
            index=models.Index(fields=['mfg', 'mfg_pn'], name='epic_part_mfg_e03baa_idx'),
        ),
        migrations.AddIndex(
            model_name='line_item',
            index=models.Index(fields=['txtn', 'index'], name='epic_line_i_txtn_id_97b974_idx'),
        ),
    ]
