# Generated by Django 3.2.14 on 2022-07-26 00:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epic', '0011_auto_20220725_0413'),
    ]

    operations = [
        migrations.AlterField(
            model_name='part',
            name='footprint',
            field=models.CharField(blank=True, default='', help_text="The part's PCB footprint.", max_length=127),
        ),
    ]
