.epic_deleted {
    text-decoration: line-through;
    color: red;
}

.epic_ok {
    color: green;
}

.epic_alert {
    color: red;
}

.epic_excess {
    color: green;
}

.epic_shortage {
    color: red;
}

.epic_substitute_deck {
    height: inherit;
}

.epic_substitute_element, .epic_substitute_add {
    border: 1px solid #ccc;
    border-radius: 4px;
    padding: 6px;
}

.epic_substitute_add input {
    border: 1px solid #ccc;
    border-radius: 4px;
}

.date {
    width: 10em;
}

.center {
    text-align: center;
}

/* Crispy-forms adds the "form-control" class to multi-value fields
   which limits the height of the control to 36px but that can cause
   multi-valued select2 controls to overflow the available area,
   causing ugly overlaps. */
.select2 .select2-selection--multiple {
    height: auto;
}
